package meerkat.voting.controller;

import meerkat.protobuf.Voting.*;
import meerkat.voting.controller.selector.QuestionSelector;
import meerkat.voting.encryptor.VBCryptoManager;
import meerkat.voting.ui.VotingBoothUI;
import meerkat.voting.output.BallotOutputDevice;
import meerkat.voting.storage.StorageManager;

import java.util.List;


/**
 * An interface for the controller component of the voting booth
 */
public interface VotingBoothController extends Runnable{

    /**
     * initialize by setting all the different components of the Voting Booth to be recognized by this controller
     * @param outputDevice the ballot output device. Naturally a printer and/or ethernet connection
     * @param vbCrypto the crypto module
     * @param vbUI User interface in which the voter chooses his answers
     * @param vbStorageManager storage component for handling files and USB sticks
     */
    public void init (BallotOutputDevice outputDevice,
                      VBCryptoManager vbCrypto,
                      VotingBoothUI vbUI,
                      StorageManager vbStorageManager);

    /**
     * set the voting questions
     * @param questions
     */
    public void setBallotChannelChoiceQuestions(List<BallotQuestion> questions);

    /**
     * Set the channel question-selector (the component which matches the ballot questions to each user)
     * @param selector the question selector instance
     */
    public void setChannelQuestionSelector (QuestionSelector selector);

    /**
     * set the voting race questions
     * @param questions
     */
    public void setBallotRaceQuestions(List<BallotQuestion> questions);

    /**
     * an asynchronous call from Admin Console (If there is such one implemented) to shut down the system
     */
    public void callShutDown();


}
