package meerkat.voting.controller.callbacks;

import meerkat.protobuf.Voting.*;
import meerkat.voting.controller.SystemMessages;
import meerkat.voting.controller.commands.ControllerCommand;
import meerkat.voting.controller.commands.EncryptAndCommitBallotCommand;
import meerkat.voting.controller.commands.ReportErrorCommand;
import meerkat.voting.controller.commands.RestartVotingCommand;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.concurrent.LinkedBlockingQueue;

public class VotingCallback extends ControllerCallback<List<BallotAnswer>> {
    protected final static Logger logger = LoggerFactory.getLogger(VotingCallback.class);

    public VotingCallback(int requestId,
                          long ballotSerialNumber,
                          LinkedBlockingQueue<ControllerCommand> controllerQueue) {
        super(requestId, ballotSerialNumber, controllerQueue);
    }

    @Override
    public void onSuccess(List<BallotAnswer> result) {
        logger.debug("callback for voting returned success");
        enqueueCommand(new EncryptAndCommitBallotCommand(getRequestIdentifier(), getBallotSerialNumber(), result));
    }

    @Override
    public void onFailure(Throwable t) {
        if (t instanceof VoterCancelException) {
            logger.debug("VotingCallback got a cancellation response");
            enqueueCommand(new RestartVotingCommand(getRequestIdentifier(), getBallotSerialNumber()));
        }
        else {
            logger.error("voting initiated a failure: " + t);
            enqueueCommand(new ReportErrorCommand(getRequestIdentifier(),
                    getBallotSerialNumber(),
                    SystemMessages.getUnsuccessfulVotingMessage()));
        }
    }
}
